@@ -23,22 +23,29 @@
 
 #include <QHash>
 #include <QWidget>
-#include <QDBusConnection>
 #include <QPointer>
 #include <QBuffer>
 #include <QFileInfo>
-#include <QDBusConnectionInterface>
 #include <KPluginLoader>
 #include <KPluginMetaData>
 
+#ifdef QT_DBUS_LIB
+#include <QDBusConnection>
+#include <QDBusConnectionInterface>
+#endif
+
 #include "knotifyconfig.h"
 #include "knotificationplugin.h"
-#include "notifybypopup.h"
 
 #include "notifybylogfile.h"
 #include "notifybytaskbar.h"
 #include "notifybyexecute.h"
-#include "notifybyflatpak.h"
+#ifndef Q_OS_ANDROID
+#include "notifybypopup.h"
+#include "notifybyportal.h"
+#else
+#include "notifybyandroid.h"
+#endif
 #include "debug_p.h"
 
 #if defined(HAVE_CANBERRA)
@@ -60,7 +67,6 @@ struct Q_DECL_HIDDEN KNotificationManager::Private {
     // incremental ids for notifications
     int notifyIdCounter;
     QStringList dirtyConfigCache;
-    bool inSandbox = false;
     bool portalDBusServiceExists = false;
 };
 
@@ -84,14 +90,10 @@ KNotificationManager::KNotificationManager()
     qDeleteAll(d->notifyPlugins);
     d->notifyPlugins.clear();
 
-    if (!qEnvironmentVariableIsEmpty("XDG_RUNTIME_DIR")) {
-        const QByteArray runtimeDir = qgetenv("XDG_RUNTIME_DIR");
-        if (!runtimeDir.isEmpty()) {
-            d->inSandbox = QFileInfo::exists(QFile::decodeName(runtimeDir) + QLatin1String("/flatpak-info"));
-        }
-    }
+#ifdef QT_DBUS_LIB
+    const bool inSandbox = QFileInfo::exists(QLatin1String("/.flatpak-info")) || qEnvironmentVariableIsSet("SNAP");
 
-    if (d->inSandbox) {
+    if (inSandbox) {
         QDBusConnectionInterface *interface = QDBusConnection::sessionBus().interface();
         d->portalDBusServiceExists = interface->isServiceRegistered(QStringLiteral("org.freedesktop.portal.Desktop"));
     }
@@ -102,6 +104,7 @@ KNotificationManager::KNotificationManager()
                                           QStringLiteral("reparseConfiguration"),
                                           this,
                                           SLOT(reparseConfiguration(QString)));
+#endif
 }
 
 KNotificationManager::~KNotificationManager()
@@ -130,11 +133,15 @@ KNotificationPlugin *KNotificationManager::pluginForAction(const QString &action
     // We have a series of built-ins up first, and fall back to trying
     // to instantiate an externally supplied plugin.
     if (action == QLatin1String("Popup")) {
-            if (d->inSandbox && d->portalDBusServiceExists) {
-                plugin = new NotifyByFlatpak(this);
+#ifndef Q_OS_ANDROID
+            if (d->portalDBusServiceExists) {
+                plugin = new NotifyByPortal(this);
             } else {
                 plugin = new NotifyByPopup(this);
             }
+#else
+        plugin = new NotifyByAndroid(this);
+#endif
 
         addPlugin(plugin);
     } else if (action == QLatin1String("Taskbar")) {
@@ -192,7 +199,7 @@ KNotificationPlugin *KNotificationManager::pluginForAction(const QString &action
             },
             this);
 
-        Q_FOREACH (QObject *pluginObj, plugins) {
+        for (QObject *pluginObj : qAsConst(plugins)) {
             KNotificationPlugin *notifyPlugin = qobject_cast<KNotificationPlugin*>(pluginObj);
 
             if (notifyPlugin) {
@@ -265,7 +272,8 @@ void KNotificationManager::close(int id, bool force)
         KNotifyConfig notifyConfig(n->appName(), n->contexts(), n->eventId());
         QString notifyActions = notifyConfig.readEntry(QStringLiteral("Action"));
 
-        Q_FOREACH (const QString &action, notifyActions.split(QLatin1Char('|'))) {
+        const auto listActions = notifyActions.split(QLatin1Char('|'));
+        for (const QString &action : listActions) {
             if (!d->notifyPlugins.contains(action)) {
                 qCDebug(LOG_KNOTIFICATIONS) << "No plugin for action" << action;
                 continue;
@@ -296,7 +304,22 @@ int KNotificationManager::notify(KNotification *n)
 
     d->notifications.insert(d->notifyIdCounter, n);
 
-    Q_FOREACH (const QString &action, notifyActions.split(QLatin1Char('|'))) {
+    // TODO KF6 d-pointer KNotifyConfig and add this there
+    if (n->urgency() == KNotification::DefaultUrgency) {
+        const QString urgency = notifyConfig.readEntry(QStringLiteral("Urgency"));
+        if (urgency == QLatin1String("Low")) {
+            n->setUrgency(KNotification::LowUrgency);
+        } else if (urgency == QLatin1String("Normal")) {
+            n->setUrgency(KNotification::NormalUrgency);
+        } else if (urgency == QLatin1String("High")) {
+            n->setUrgency(KNotification::HighUrgency);
+        } else if (urgency == QLatin1String("Critical")) {
+            n->setUrgency(KNotification::CriticalUrgency);
+        }
+    }
+
+    const auto actionsList = notifyActions.split(QLatin1Char('|'));
+    for (const QString &action : actionsList) {
         KNotificationPlugin *notifyPlugin = pluginForAction(action);
 
         if (!notifyPlugin) {
@@ -317,7 +340,7 @@ void KNotificationManager::update(KNotification *n)
 {
     KNotifyConfig notifyConfig(n->appName(), n->contexts(), n->eventId());
 
-    Q_FOREACH (KNotificationPlugin *p, d->notifyPlugins) {
+    for (KNotificationPlugin *p : qAsConst(d->notifyPlugins)) {
         p->update(n, &notifyConfig);
     }
 }
