@@ -151,7 +151,7 @@ public:
      */
     class HtmlEntityResolver : public QXmlStreamEntityResolver
     {
-        QString resolveUndeclaredEntity(const QString &name) Q_DECL_OVERRIDE;
+        QString resolveUndeclaredEntity(const QString &name) override;
     };
 
 };
@@ -181,8 +181,8 @@ NotifyByPopup::NotifyByPopup(QObject *parent)
     watcher->setConnection(QDBusConnection::sessionBus());
     watcher->setWatchMode(QDBusServiceWatcher::WatchForOwnerChange);
     watcher->addWatchedService(QString::fromLatin1(dbusServiceName));
-    connect(watcher, SIGNAL(serviceOwnerChanged(QString,QString,QString)),
-            SLOT(onServiceOwnerChanged(QString,QString,QString)));
+    connect(watcher, &QDBusServiceWatcher::serviceOwnerChanged,
+            this, &NotifyByPopup::onServiceOwnerChanged);
 
 #ifndef Q_WS_WIN
     if (!d->dbusServiceExists) {
@@ -203,7 +203,7 @@ NotifyByPopup::NotifyByPopup(QObject *parent)
 
 NotifyByPopup::~NotifyByPopup()
 {
-    Q_FOREACH (KPassivePopup *p, d->passivePopups) {
+    for (KPassivePopup *p : qAsConst(d->passivePopups)) {
         p->deleteLater();
     }
 
@@ -218,7 +218,7 @@ void NotifyByPopup::notify(KNotification *notification, KNotifyConfig *notifyCon
 void NotifyByPopup::notify(KNotification *notification, const KNotifyConfig &notifyConfig)
 {
     if (d->passivePopups.contains(notification) || d->galagoNotifications.contains(notification->id())) {
-        // notification is alrady on the screen, do nothing
+        // notification is already on the screen, do nothing
         finish(notification);
         return;
     }
@@ -233,7 +233,7 @@ void NotifyByPopup::notify(KNotification *notification, const KNotifyConfig &not
             d->queryPopupServerCapabilities();
         } else {
             if (!d->sendNotificationToGalagoServer(notification, notifyConfig)) {
-                finish(notification); //an error ocurred.
+                finish(notification); //an error occurred.
             }
         }
         return;
@@ -291,7 +291,7 @@ void NotifyByPopup::notify(KNotification *notification, const KNotifyConfig &not
         d->nextPosition = screen.top();
     }
     pop->setAutoDelete(true);
-    connect(pop, SIGNAL(destroyed()), this, SLOT(onPassivePopupDestroyed()));
+    connect(pop, &QObject::destroyed, this, &NotifyByPopup::onPassivePopupDestroyed);
 
     pop->setTimeout(timeout);
     pop->adjustSize();
@@ -325,14 +325,15 @@ void NotifyByPopup::onPassivePopupDestroyed()
 void NotifyByPopup::timerEvent(QTimerEvent *event)
 {
     if (event->timerId() != d->animationTimer) {
-        return KNotificationPlugin::timerEvent(event);
+        KNotificationPlugin::timerEvent(event);
+        return;
     }
 
     bool cont = false;
     QRect screen = QGuiApplication::primaryScreen()->availableGeometry();
     d->nextPosition = screen.top();
 
-    Q_FOREACH (KPassivePopup *popup, d->passivePopups)
+    for (KPassivePopup *popup : qAsConst(d->passivePopups))
     {
         int y = popup->pos().y();
         if (y > d->nextPosition) {
@@ -414,13 +415,15 @@ void NotifyByPopup::onServiceOwnerChanged(const QString &serviceName, const QStr
 {
     Q_UNUSED(serviceName);
     // close all notifications we currently hold reference to
-    Q_FOREACH (KNotification *n, d->galagoNotifications.values()) {
+    for (KNotification *n : qAsConst(d->galagoNotifications)) {
         if (n) {
-            finished(n);
+            emit finished(n);
         }
     }
-    Q_FOREACH (KNotification *n, d->passivePopups.keys()) {
-        finished(n);
+    QMap<KNotification*, KPassivePopup *>::const_iterator i = d->passivePopups.constBegin();
+    while (i != d->passivePopups.constEnd()) {
+        emit finished(i.key());
+        ++i;
     }
     d->galagoNotifications.clear();
     d->passivePopups.clear();
@@ -488,7 +491,7 @@ void NotifyByPopup::onGalagoNotificationClosed(uint dbus_id, uint reason)
     d->galagoNotifications.remove(dbus_id);
 
     if (n) {
-        finished(n);
+        emit finished(n);
         // The popup bubble is the only user facing part of a notification,
         // if the user closes the popup, it means he wants to get rid
         // of the notification completely, including playing sound etc
@@ -594,7 +597,8 @@ void NotifyByPopupPrivate::fillPopup(KPassivePopup *popup, KNotification *notifi
     if (!notification->actions().isEmpty()) {
         QString linkCode = QStringLiteral("<p align=\"right\">");
         int i = 0;
-        Q_FOREACH (const QString &it, notification->actions()) {
+        const auto actionList = notification->actions();
+        for (const QString &it : actionList) {
             i++;
             linkCode += QStringLiteral("&nbsp;<a href=\"%1/%2\">%3</a>").arg(QString::number(notification->id()), QString::number(i), it.toHtmlEscaped());
         }
@@ -604,8 +608,10 @@ void NotifyByPopupPrivate::fillPopup(KPassivePopup *popup, KNotification *notifi
         link->setTextInteractionFlags(Qt::LinksAccessibleByMouse);
         link->setOpenExternalLinks(false);
         //link->setAlignment( AlignRight );
-        QObject::connect(link, SIGNAL(linkActivated(const QString &)), q, SLOT(onPassivePopupLinkClicked(const QString& ) ) );
-        QObject::connect(link, SIGNAL(linkActivated(const QString &)), popup, SLOT(hide()));
+        QObject::connect(link, &QLabel::linkActivated,
+                         q, &NotifyByPopup::onPassivePopupLinkClicked);
+        QObject::connect(link, &QLabel::linkActivated,
+                         popup, &QWidget::hide);
     }
 
     popup->setView( vb );
@@ -668,7 +674,8 @@ bool NotifyByPopupPrivate::sendNotificationToGalagoServer(KNotification *notific
             actionList.append(defaultAction);
         }
         int actId = 0;
-        Q_FOREACH (const QString &actionName, notification->actions()) {
+        const auto listActions = notification->actions();
+        for (const QString &actionName : listActions) {
             actId++;
             actionList.append(QString::number(actId));
             actionList.append(actionName);
@@ -701,8 +708,41 @@ bool NotifyByPopupPrivate::sendNotificationToGalagoServer(KNotification *notific
         hintsMap[QStringLiteral("transient")] = true;
     }
 
-    if (!qApp->desktopFileName().isEmpty()) {
-        hintsMap[QStringLiteral("desktop-entry")] = qApp->desktopFileName();
+    QString desktopFileName = QGuiApplication::desktopFileName();
+    if (!desktopFileName.isEmpty()) {
+        // handle apps which set the desktopFileName property with filename suffix,
+        // due to unclear API dox (https://bugreports.qt.io/browse/QTBUG-75521)
+        if (desktopFileName.endsWith(QLatin1String(".desktop"))) {
+            desktopFileName.chop(8);
+        }
+        hintsMap[QStringLiteral("desktop-entry")] = desktopFileName;
+    }
+
+    int urgency = -1;
+    switch (notification->urgency()) {
+    case KNotification::DefaultUrgency:
+        break;
+    case KNotification::LowUrgency:
+        urgency = 0;
+        break;
+    case KNotification::NormalUrgency:
+        Q_FALLTHROUGH();
+    // galago notifications only know low, normal, critical
+    case KNotification::HighUrgency:
+        urgency = 1;
+        break;
+    case KNotification::CriticalUrgency:
+        urgency = 2;
+        break;
+    }
+
+    if (urgency > -1) {
+        hintsMap[QStringLiteral("urgency")] = urgency;
+    }
+
+    const QVariantMap hints = notification->hints();
+    for (auto it = hints.constBegin(); it != hints.constEnd(); ++it) {
+        hintsMap[it.key()] = it.value();
     }
 
     //FIXME - reenable/fix
@@ -732,8 +772,8 @@ bool NotifyByPopupPrivate::sendNotificationToGalagoServer(KNotification *notific
     QDBusPendingCallWatcher *watcher = new QDBusPendingCallWatcher(notificationCall, notification);
     watcher->setProperty("notificationObject", QVariant::fromValue<KNotification*>(notification));
 
-    QObject::connect(watcher, SIGNAL(finished(QDBusPendingCallWatcher*)),
-                     q, SLOT(onGalagoServerReply(QDBusPendingCallWatcher*)));
+    QObject::connect(watcher, &QDBusPendingCallWatcher::finished,
+                     q, &NotifyByPopup::onGalagoServerReply);
 
     return true;
 }
